#from FTDock import *
from HEXDock import *
from ZDock import *
from PATCHDock import *
from SplitPotentials import SplitPotentialsMatrix
from SplitPotentialsPPI import *
from ScoringFunctions import *
